/**
  ******************************************************************************
  * @file    system_tpum33xx.c
  * @author  3peak Asic Team
  * @brief   CMSIS Device Peripheral Access Layer System Source File.
  *          This file contains the system clock configuration for devices.
  ******************************************************************************
  * @attention
  *
  * Copyright (c) 2022 3Peak Microelectronics.
  * All rights reserved.
  *
  * SPDX-License-Identifier: BSD-3-Clause.
  *
  ******************************************************************************
  */
#include <stddef.h>
#include <math.h>
#include "tps325mx.h"

/*----------------------------------------------------------------------------
  Define SYSCLK
 *----------------------------------------------------------------------------*/
#define __IHS (8000000UL)

#define RCC_SYSCLK_DIV_1                (0x00UL << RCC_CFGR_HPRE_Pos)    /*!< SYSCLK not divided */
#define RCC_SYSCLK_DIV_2                (0x08UL << RCC_CFGR_HPRE_Pos)    /*!< SYSCLK divided by 2 */
#define RCC_SYSCLK_DIV_4                (0x09UL << RCC_CFGR_HPRE_Pos)    /*!< SYSCLK divided by 4 */
#define RCC_SYSCLK_DIV_8                (0x0AUL << RCC_CFGR_HPRE_Pos)    /*!< SYSCLK divided by 8 */
#define RCC_SYSCLK_DIV_16               (0x0BUL << RCC_CFGR_HPRE_Pos)    /*!< SYSCLK divided by 16 */
#define RCC_SYSCLK_DIV_64               (0x0CUL << RCC_CFGR_HPRE_Pos)    /*!< SYSCLK divided by 64 */
#define RCC_SYSCLK_DIV_128              (0x0DUL << RCC_CFGR_HPRE_Pos)    /*!< SYSCLK divided by 128 */
#define RCC_SYSCLK_DIV_256              (0x0EUL << RCC_CFGR_HPRE_Pos)    /*!< SYSCLK divided by 256 */
#define RCC_SYSCLK_DIV_512              (0x0FUL << RCC_CFGR_HPRE_Pos)    /*!< SYSCLK divided by 512 */

#define RCC_SYSCLK_DIV                  RCC_SYSCLK_DIV_1                 /*!<Set SYSCLK divided   */

/** @defgroup RCC_PDL_EC_APB1_DIV  APB low-speed prescaler (APB1)
    * @{
    */
#define RCC_APB1_DIV_1                  (0x00UL << RCC_CFGR_PPRE1_Pos)   /*!< HCLK not divided */
#define RCC_APB1_DIV_2                  (0x04UL << RCC_CFGR_PPRE1_Pos)   /*!< HCLK divided by 2 */
#define RCC_APB1_DIV_4                  (0x05UL << RCC_CFGR_PPRE1_Pos)   /*!< HCLK divided by 4 */
#define RCC_APB1_DIV_8                  (0x06UL << RCC_CFGR_PPRE1_Pos)   /*!< HCLK divided by 8 */
#define RCC_APB1_DIV_16                 (0x07UL << RCC_CFGR_PPRE1_Pos)   /*!< HCLK divided by 16 */

/** @defgroup RCC_PDL_EC_APB2_DIV  APB high-speed prescaler (APB2)
    * @{
    */
#define RCC_APB2_DIV_1                  (0x00UL << RCC_CFGR_PPRE2_Pos)   /*!< HCLK not divided */
#define RCC_APB2_DIV_2                  (0x04UL << RCC_CFGR_PPRE2_Pos)   /*!< HCLK divided by 2 */
#define RCC_APB2_DIV_4                  (0x05UL << RCC_CFGR_PPRE2_Pos)   /*!< HCLK divided by 4 */
#define RCC_APB2_DIV_8                  (0x06UL << RCC_CFGR_PPRE2_Pos)   /*!< HCLK divided by 8 */
#define RCC_APB2_DIV_16                 (0x07UL << RCC_CFGR_PPRE2_Pos)   /*!< HCLK divided by 16 */

/** @defgroup RCC_PDL_EC_SYS_CLKSOURCE  System clock switch
    * @{
    */
#define RCC_SYS_CLKSOURCE_IHS           (0x00UL << RCC_CFGR_SW_Pos)    /*!< IHS selection as system clock */
#define RCC_SYS_CLKSOURCE_EHS           (0x01UL << RCC_CFGR_SW_Pos)    /*!< EHS selection as system clock */
#define RCC_SYS_CLKSOURCE_PLL           (0x02UL << RCC_CFGR_SW_Pos)    /*!< PLL selection as system clock */

/** @defgroup RCC_PDL_EC_SYS_CLKSOURCE_STATUS  System clock switch status
    * @{
    */
#define RCC_SYS_CLKSOURCE_STATUS_IHS    (0x00UL << RCC_CFGR_SWS_Pos)   /*!< IHS used as system clock */
#define RCC_SYS_CLKSOURCE_STATUS_EHS    (0x01UL << RCC_CFGR_SWS_Pos)   /*!< EHS used as system clock */
#define RCC_SYS_CLKSOURCE_STATUS_PLL    (0x02UL << RCC_CFGR_SWS_Pos)   /*!< PLL used as system clock */

#if !defined  (EHS_VALUE)
#define EHS_VALUE    8000000U   /*!< Value of the EHS oscillator in Hz */
#endif /* EHS_VALUE */

#if !defined  (IHS_VALUE)
#define IHS_VALUE    8000000U  /*!< Value of the IHS oscillator in Hz */
#endif /* IHS_VALUE */

#if !defined  (ELS_VALUE)
#define ELS_VALUE    32768U     /*!< Value of the ELS oscillator in Hz */
#endif /* ELS_VALUE */

#if !defined  (ILS_VALUE)
#define ILS_VALUE    32000U     /*!< Value of the ILS oscillator in Hz */
#endif /* ILS_VALUE */

/*----------------------------------------------------------------------------
  Clock Definitions
 *----------------------------------------------------------------------------*/
//const uint8_t AHBPrescTable[16] = {0U, 0U, 0U, 0U, 0U, 0U, 0U, 0U, 1U, 2U, 3U, 4U, 6U, 7U, 8U, 9U};
//const uint8_t APBPrescTable[8] =  {0U, 0U, 0U, 0U, 1U, 2U, 3U, 4U};

uint32_t SystemCoreClock = 8000000UL;                   /*!< System Clock Frequency (Core Clock) */
extern uint32_t g_delayFreqMhz;
extern uint32_t g_delayFreqKhz;

/* #define VECT_TAB_SRAM */
#define VECT_TAB_OFFSET  0x0U /*!< Vector Table base offset field.
                                   This value must be a multiple of 0x200. */

/* SCB->CPACR */
#define SCB_CPACR_CP10_CP11_ENABLE      (0xFUL << 20u)

/**
  * @brief  Setup the microcontroller system.
  * @param  none
  * @retval none
  */
void SystemInit (void)
{
#if (__FPU_PRESENT == 1) && (__FPU_USED == 1)
    /* set CP10 and CP11 Full Access */
    SCB->CPACR |= ((3UL << 10*2) | (3UL << 11*2));
    __DSB();
    __ISB();
#endif

    SCB_EnableICache();
    //SCB_EnableDCache();

    //EMSS->ITCMCR |= SCB_ITCMCR_EN_Msk;
    //EMSS->DTCMCR |= SCB_DTCMCR_EN_Msk;

    /* Enable flash auto-gating by default, this is to allow successful entry into low power mode */
    FMC->ACR |= FMC_ACR_WCLKMODE_Msk;

    /* Enable PMU clock gate by default */
    RCC->APB1ENR |= RCC_APB1ENR_PMUEN_Msk;

    /* 
     * By default, SRAM0 and SRAM1 retention bits are 0 which powers down SRAM0 and SRAM1 under Stop Mode.
     * Enable all SRAM retention bits to allow retention of data stored in entire SRAM  
     */
    PMU->CR01 |= PMU_CR01_STOPSRAM0RET0_Msk | PMU_CR01_STOPSRAM0RET1_Msk | PMU_CR01_STOPSRAM0RET2_Msk |
                 PMU_CR01_STOPSRAM1RET0_Msk | PMU_CR01_STOPSRAM1RET1_Msk | PMU_CR01_STOPSRAM2RET_Msk;

    /* Configure the Vector Table location add offset address ------------------*/
#ifdef VECT_TAB_SRAM
    SCB->VTOR = RAM_BASE | VECT_TAB_OFFSET;    /* Vector Table Relocation in Internal SRAM */
#else
    SCB->VTOR = FLASH_BASE | VECT_TAB_OFFSET;  /* Vector Table Relocation in Internal FLASH */
#endif
}

void SystemCoreClockUpdate(void)
{
    uint32_t pllinputfreq;
    uint32_t systemclocksource;
    uint32_t pllsource;
    uint32_t plloutfreq;
    uint32_t indiv;
    uint32_t fbdiv; 
    uint32_t odiv;
    const uint32_t const_power[4] = {1,2,4,8};

    systemclocksource = ((RCC->CFGR & RCC_CFGR_SWS_Msk) >> RCC_CFGR_SWS_Pos);
    
    switch (systemclocksource)
    {
        case 0x00:  /* IHS used as PLL clock source */
        {
            SystemCoreClock = IHS_VALUE;
        }
        break;

        case 0x01:  /* EHS used as PLL clock source */
        {
            SystemCoreClock = EHS_VALUE;
        }
        break;

        case 0x02:  /* EHS used as PLL clock source */
        {
            /* PLL_VCO = (EHS_VALUE or IHS_VALUE / Indiv) * Fbdiv
               SYSCLK = PLL_VCO / Odiv
            */
            pllsource = (RCC->PLLCFGR1 & RCC_PLLCFGR1_PLLSRC_Msk);
            
            if(pllsource == 0x02)
            {
                pllinputfreq = IHS_VALUE;
            }
            else
            {
                pllinputfreq = EHS_VALUE;
            }
            
            indiv = const_power[((RCC->PLLCFGR1 & RCC_PLLCFGR1_PLLINDIV_Msk) >> RCC_PLLCFGR1_PLLINDIV_Pos)];
            fbdiv = ((RCC->PLLCFGR1 & RCC_PLLCFGR1_PLLFBDIV_Msk) >> RCC_PLLCFGR1_PLLFBDIV_Pos);
            odiv = const_power[(((RCC->PLLCFGR1 & RCC_PLLCFGR1_PLLODIV_Msk) >> RCC_PLLCFGR1_PLLODIV_Pos)+1)];
            plloutfreq = (pllinputfreq/indiv)*fbdiv/odiv;
            SystemCoreClock = plloutfreq;
        }
        break;
        
        default:
        {
            SystemCoreClock = IHS_VALUE;
        }
        break;
    }
    
    g_delayFreqKhz = SystemCoreClock/1000;
    g_delayFreqMhz = SystemCoreClock/1000000;
}
